@@ -25,7 +25,7 @@
   utils_init(argv); \
 } while (0)
 
-#define OPT(option, desc) printf("\t" option "%s\n", desc)
+#define OPT(option, desc) printf("  " option "%s\n", desc)
 
 #define OPTIONS_ARE() printf(_("valid options are:\n"))
 #define COMMANDS_ARE() printf(_("valid commands are:\n"))
