@@ -8,6 +8,7 @@ typedef enum { False=0, True=1 } lct_boolean;
 struct magic
 {
   int id;
+  char* format_name;
   char* magic;
   char* mask;
   size_t length;
@@ -36,12 +37,12 @@ FILE* findfile(const char *fnam, const char **dirpath, const char **suffixes,
 FILE* find ## NAME(const char *fnam, char *fullname, size_t maxfullength, \
                    FILE* minus_meaning, int* magic_return)
 
-FINDFILE_WRAPPER_HEADER(font);
-FINDFILE_WRAPPER_HEADER(acm);
-FINDFILE_WRAPPER_HEADER(sfm);
-FINDFILE_WRAPPER_HEADER(sfmfallback);
-FINDFILE_WRAPPER_HEADER(keymap);
-FINDFILE_WRAPPER_HEADER(videomode);
+/* template for wrappers around findfont */
+#define FINDFILE_WRAPPER(NAME, PATH, SUFFIXES, MAGICS) \
+FINDFILE_WRAPPER_HEADER(NAME) \
+{ \
+  return findfile(fnam, PATH, SUFFIXES, fullname, maxfullength, minus_meaning, MAGICS, magic_return); \
+}
 
 /* from x*.c */
 int xfseek (FILE *stream, long offset, int whence);
