@@ -3,13 +3,22 @@
 
 #include <stdio.h>
 #include <linux/kd.h>
+
 #include <lct/unicode.h>
+#include <lct/generic.h>
 
 #if 0
 /* just an idea ... */
 typedef enum { LCT_OK, LCT_ERRNO, LCT_ENOTYET } lct_errcode;
 #endif
 
+
+FINDFILE_WRAPPER_HEADER(acm);
+FINDFILE_WRAPPER_HEADER(sfm);
+FINDFILE_WRAPPER_HEADER(sfmfallback);
+FINDFILE_WRAPPER_HEADER(keymap);
+FINDFILE_WRAPPER_HEADER(videomode);
+
 /* from sfm-utils.c */
 unicode sgetunicode(char **p0);
 void unimapdesc_addpair(int fp, unicode un, struct unimapdesc *list, int *listsz);
