@@ -1,7 +1,6 @@
 
 /*  A Bison parser, made from loadkeys.y
- by  GNU Bison version 1.27
-  */
+    by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
 
@@ -305,7 +304,7 @@ static const short yycheck[] = {     3,
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
 #line 3 "/usr/share/misc/bison.simple"
-/* This file comes from bison-1.27.  */
+/* This file comes from bison-1.28.  */
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -518,7 +517,7 @@ __yy_memcpy (char *to, char *from, unsigned int count)
 #endif
 #endif
 
-#line 216 "/usr/share/misc/bison.simple"
+#line 217 "/usr/share/misc/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -1029,7 +1028,7 @@ case 47:
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 542 "/usr/share/misc/bison.simple"
+#line 543 "/usr/share/misc/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1885,12 +1884,17 @@ static void loadkeys (void)
     diacct = defdiacs(fd);
   if (verbose)
     {
-      printf(_("\nChanged %d key%s and %d string%s.\n"),
-	     keyct, (keyct == 1) ? "" : "s",
-	     funcct, (funcct == 1) ? "" : "s");
+      char* nbuf = NULL;
+      unsigned nbufsize = 0;
+      
+      printf(_("\nChanged %s and %s.\n"),
+	       LCT_NUMBER(nbuf, nbufsize, keyct, "key", "keys"),
+	       LCT_NUMBER(nbuf, nbufsize, funcct, "string", "strings"));
       if (accent_table_size > 0 || nocompose)
-	printf(_("Loaded %d compose definition%s.\n"),
-	       diacct, (diacct == 1) ? "" : "s");
+	printf(_("Loaded %s.\n"),
+	       LCT_NUMBER(nbuf, nbufsize, diacct,
+		      "compose definition",
+		      "compose definitions"));
       else
 	printf(_("(No change in compose definitions.)\n"));
     }
