@@ -31,6 +31,8 @@
 
 #define DEFUNIMAP DATADIR "/" TRANSDIR "/def"
 
+const int DEFAULT_FFF = FFF_PSF;
+
 int verbose = 0;
 int no_act = 0;
 
@@ -49,33 +51,35 @@ static void usage()
   version();
   printf(_("Usage:  %s [options] [commands]\n"), progname);
   OPTIONS_ARE();
-  OPT(" -v --verbose        ", _("List operations as they are done"));
-  OPT(" -n --no-act         ", _("Do not change the console state nor write to any file"));
-  OPT(" -H --char-height=N  ", _("(N in 0..32) Choose the right font from a codepage that\n"
+  OPT("-v --verbose        ", _("List operations as they are done"));
+  OPT("-n --no-act         ", _("Do not change the console state nor write to any file"));
+  OPT("-H --char-height=N  ", _("(N in 0..32) Choose the right font from a codepage that\n"
 				 "contains three fonts (only 8/14/16 allowed then), or choose\n"
 				 "default font, ie. \"default8xN\""));
-  OPT(" --force-no-sfm      ", _("Suppress loading of a screen-font map [use with care]"));
-  OPT(" -1 --g1             ", _("When loading an ACM, activate G1 charset instead of G0"));
-  OPT(" --tty=device        ", _("Use `device' as console device for ioctls"));
-  COMMANDS_ARE();
-  OPT(" -f --font=file      ", _("Load the console-font from specified file"));
-  OPT(" -d --default-font   ", _("Load a default font from a file"));
-  OPT(" -R --rom-font       ", _("Restore ROM font (does not work with all kernels)"));
-  OPT(" -u --sfm --screen-font-map=file", "");
-  OPT("                     ", _("Load the SFM from specified file\n"
-				 "(instead of the one in font-file, if any)"));
-  OPT(" -k --sfm-fallback=file  ", _("Merge SFM fallbacks from file into SFM"));
-  OPT(" -m --acm --app-charset-map=file  ", _("Load the ACM from specified file"));
-  OPT(" -F --old-font=file  ", _("Write current font to prefered format (now: psf-with-sfm)"));
-  OPT(" --old-font-psf=file ", _("Write current font to PSF file before loading a new one"));
-  OPT(" --old-font-psf-with-sfm=file", "");
-  OPT("                     ", _("Same as -old-font-psf, and add current SFM in the PSF file"));
-  OPT(" --old-font-raw=file ", _("Write current font to RAW file before loading a new one"));
-  OPT(" -M --old-acm=file   ", _("Write current ACM to file before loading a new one"));
-  OPT(" -U --old-sfm=file   ", _("Write current SFM to file before loading a new one"));
+  OPT("--force-no-sfm      ", _("Suppress loading of a screen-font map [use with care]"));
+  OPT("-1 --g1             ", _("When loading an ACM, activate G1 charset instead of G0"));
+  OPT("--tty=device        ", _("Use `device' as console device for ioctls"));
 
-  OPT("-h --help            ", HELPDESC);
-  OPT("-V --version         ", VERSIONDESC);
+  COMMANDS_ARE();
+  OPT("-f --font=file      ", _("Load the console-font from specified file"));
+  OPT("-d --default-font   ", _("Load a default font from a file"));
+  OPT("-R --rom-font       ", _("Restore ROM font (does not work with all kernels)"));
+  OPT("-u --sfm --screen-font-map=file", "");
+  OPT("                    ", _("Load the SFM from specified file\n"
+				 "(instead of the one in font-file, if any)"));
+  OPT("-k --sfm-fallback=file  ", _("Merge SFM fallbacks from file into SFM"));
+  OPT("-m --acm --app-charset-map=file  ", _("Load the ACM from specified file"));
+  OPT("-F --old-font=file  ", _("Write current font to prefered format (now: psf-with-sfm)"));
+/*    OPT("--old-font-psf=file ", _("Write current font to PSF file before loading a new one")); */
+/*    OPT("--old-font-raw=file ", _("Write current font to RAW file before loading a new one")); */
+  OPT("   --font-format=.. ", _("Select font file format for saving"));
+  OPT("   --list-font-formats", "");
+  OPT("                    ", _("List font file formats available to libcfont"));
+  OPT("-M --old-acm=file   ", _("Write current ACM to file before loading a new one"));
+  OPT("-U --old-sfm=file   ", _("Write current SFM to file before loading a new one"));
+
+  OPT("-h --help           ", HELPDESC);
+  OPT("-V --version        ", VERSIONDESC);
 }
 
 static inline void CHECK_STD(char* file, int* stdfile)
@@ -91,8 +95,11 @@ static inline void CHECK_STD(char* file, int* stdfile)
 
 #define OPT_or 128
 #define OPT_op 129
-#define OPT_no_sfm 130
-#define OPT_tty 131
+#define OPT_ops 130
+#define OPT_no_sfm 131
+#define OPT_tty 132
+#define OPT_fff 133
+#define OPT_fff_list 134
 
 const struct option opts[] = 
 {
@@ -114,10 +121,11 @@ const struct option opts[] =
 
   /* default old-font is synonym for old-font-psf-with-sfm */
   /* this may change when better formats are supported */
-  {"old-font-psf-with-sfm", required_argument, 0, 'F'},
   {"old-font-psf", required_argument, 0, OPT_op},
   {"old-font-raw", required_argument, 0, OPT_or},
-  
+  {"font-format", required_argument, 0, OPT_fff},
+  {"list-font-formats", no_argument, 0, OPT_fff_list},
+
   {"screen-font-map", required_argument, 0, 'u'},
   {"sfm", required_argument, 0, 'u'},
   {"app-charset-map", required_argument, 0, 'm'},
@@ -140,20 +148,21 @@ char* rom_font = (char*)(-1);
 
 void main(int argc, char *argv[])
 {
-  char *ifil, *mfil, *ufil, *orfil, *opfil, *omfil, *oufil, *opufil, **fallback_files;
+  char *ifil, *mfil, *ufil, *ofil, *omfil, *oufil, **fallback_files;
   int nb_fallback_files = 0;
   int fd = 0;
   int requested_height, no_u;
   int stdin_taken, stdout_taken;
   char* tty_name = NULL;	       /* optional name of a tty for the console */
   int g_set = 0;		       /* G0 or G1 */
+  int old_font_format = FF_UNKNOWN;
   
   struct unimapdesc ud;
   
   int result;			       /* option handling */
   int an_option;
 
-  ifil = mfil = ufil = orfil = opfil = omfil = oufil = opufil = NULL;
+  ifil = mfil = ufil = omfil = oufil = ofil = NULL;
   fallback_files = NULL;
   requested_height = 0;
   stdin_taken = stdout_taken = 0;      /* 1 means respective FILE is already used for file I/O */
@@ -166,7 +175,7 @@ void main(int argc, char *argv[])
       result = getopt_long(argc, argv, "Vhvn1H:f:F:dRu:m:U:M:k:", opts, &an_option);
 
       if (result == EOF)
-	  break;
+	break;
       
       switch (result)
 	{
@@ -193,40 +202,40 @@ void main(int argc, char *argv[])
 	  /* numeric height */
 	  requested_height = atoi(optarg);
 	  if(requested_height <= 0 || requested_height > 32)
-	      badusage(_("--char-height argument should be in 1..31"));
+	    badusage(_("--char-height argument should be in 1..31"));
 	  break;
 
 	case 'f':
 	  if (ifil)
-	      badusage(_("only one font file is allowed"));
+	    badusage(_("only one font file is allowed"));
 	  ifil = optarg;
 	  CHECK_STD(ifil, &stdin_taken);
 	  break;
 
 	case 'd':
 	  if (ifil)
-	      badusage(_("only one font file is allowed"));
+	    badusage(_("only one font file is allowed"));
 	  ifil = default_font;
 	  break;
 	  
 	case 'R':
 	  if (ifil)
-	      badusage(_("only one font file is allowed"));
+	    badusage(_("only one font file is allowed"));
 	  ifil = rom_font;
 	  break;
 	  
 	case 'm':
 	  if (mfil)
-	      badusage(_("only one ACM is allowed"));
+	    badusage(_("only one ACM is allowed"));
 	  mfil = optarg;
 	  CHECK_STD(mfil, &stdin_taken);
 	  break;
 
 	case 'u':
 	  if (ufil)
-	      badusage(_("only one SFM is allowed"));
+	    badusage(_("only one SFM is allowed"));
 	  if (no_u)
-	      badusage(_("multiple requests for SFM handling"));
+	    badusage(_("multiple requests for SFM handling"));
 	  ufil = optarg;
 	  CHECK_STD(ufil, &stdin_taken);
 	  no_u = 1;
@@ -251,36 +260,59 @@ void main(int argc, char *argv[])
 	  break;
 	  
 	case OPT_or:
-	  if (orfil)
-	      badusage(_("only one output RAW font-file is allowed"));
-	  orfil = optarg;
-	  CHECK_STD(orfil, &stdout_taken);
+	  if (ofil)
+	    badusage (_("only one output font file is allowed"));
+	  ofil = optarg;
+	  old_font_format = FFF_RAW;
+	  CHECK_STD(ofil, &stdout_taken);
 	  break;
 	  
 	case OPT_op:
-	  if (opfil)
-	      badusage(_("only one output PSF font-file is allowed"));
-	  opfil = optarg;
-	  CHECK_STD(opfil, &stdout_taken);
-	  break;
-	  
-	case 'F':
-	  if (opufil)
-	      badusage(_("only one output PSF+SFM font-file is allowed"));
-	  opufil = optarg;
-	  CHECK_STD(opufil, &stdout_taken);
+	  if (ofil)
+	    badusage (_("only one output font file is allowed"));
+	  ofil = optarg;
+	  old_font_format = FFF_PSF;
+	  CHECK_STD(ofil, &stdout_taken);
 	  break;
 
+	case 'F':
+	  if (ofil)
+	    badusage (_("only one output font file is allowed"));
+	  ofil = optarg;
+	  CHECK_STD(ofil, &stdout_taken);
+	  break;
+
+	case OPT_fff:
+	  {
+	    int tmp_fff;
+
+	    if (ofil && (old_font_format != FF_UNKNOWN))
+	      badusage (_("only one output font file is allowed"));
+
+	    tmp_fff = fff_name_to_id (optarg);
+	    if (tmp_fff == FF_UNKNOWN)
+	      badusage (_("unknown font format"));
+	    
+	    old_font_format = tmp_fff;
+	  }
+	  break;
+
+	case OPT_fff_list:
+	  printf (_("Available file formats:\n\n"));
+	  fff_fprintf_names (stdout, "\t%s\n");
+	  exit(0);
+	  break;
+	  
 	case 'M':
 	  if (omfil)
-	      badusage(_("only one output ACM file is allowed"));
+	    badusage(_("only one output ACM file is allowed"));
 	  omfil = optarg;
 	  CHECK_STD(omfil, &stdout_taken);
 	  break;
 	  
 	case 'U':
 	  if (oufil)
-	      badusage(_("only one output SFM file is allowed"));
+	    badusage(_("only one output SFM file is allowed"));
 	  oufil = optarg;
 	  CHECK_STD(oufil, &stdout_taken);
 	  break;
@@ -296,10 +328,10 @@ void main(int argc, char *argv[])
     }
 
   if (optind < argc)
-      badusage (_("no non-option arguments are valid"));
+    badusage (_("no non-option arguments are valid"));
 
-  if (!ifil && !mfil && !ufil && !orfil && !opfil && !omfil && !oufil && !opufil && !fallback_files)
-      badusage(_("nothing to do"));
+  if (!ifil && !mfil && !ufil && !ofil && !omfil && !oufil && !fallback_files)
+    badusage(_("nothing to do"));
 
   if (-1 == (fd = get_console_fd(tty_name)))
     {
@@ -311,22 +343,28 @@ void main(int argc, char *argv[])
    * save current state
    */
   
-  if (orfil)
-      if (saveoldfont(fd, orfil, 0, 0))
-	  perror (_("Saving raw old font")), exit(1);
+  if (ofil)
+    {
+      if (old_font_format == FFF_RAW)
+	{
+	  if (saveoldfont(fd, ofil, 0, 0))
+	    perror (_("Saving raw old font")), exit(1);
+	}
 
-  if (opfil)
-      if (saveoldfont(fd, opfil, 1, 0))
-	  perror (_("Saving PSF old font")), exit(1);
-
-  if (opufil)
-      if (saveoldfont(fd, opufil, 1, 1))
-	  perror (_("Saving PSF+unimap old font")), exit(1);
+      else if (old_font_format == FFF_PSF)
+	{
+	  if (saveoldfont(fd, ofil, 1, 1))
+	    perror (_("Saving PSF old font")), exit(1);
+	}
 
+      else
+	fprintf (stderr, _("Cannot save font using this file format.\n")), exit(1);
+    }
+  
   if (omfil)
     {
       if (no_act)
-	   fprintf (stderr, _("Would save ACM to file `%s'.\n"), omfil);
+	fprintf (stderr, _("Would save ACM to file `%s'.\n"), omfil);
       else
 	{
 	  if (verbose) fprintf (stderr, _("Saving ACM to file `%s'.\n"), omfil);
@@ -335,7 +373,7 @@ void main(int argc, char *argv[])
     }
   
   if (oufil)
-      sfm_save (fd, oufil, verbose, no_act);
+    sfm_save (fd, oufil, verbose, no_act);
   
   /*
    * set new state
@@ -347,11 +385,11 @@ void main(int argc, char *argv[])
       char pathname[1024];
       
       if ((f = findacm(mfil, pathname, sizeof(pathname), stdin, NULL)) == NULL)
-	  perror("findacm"), exit(1);
+	perror("findacm"), exit(1);
       
 
       if (no_act)
-	  printf(_("Would load ACM from `%s'\n"), pathname);
+	printf(_("Would load ACM from `%s'\n"), pathname);
       else
 	{
 	  if (verbose) printf(_("Loading ACM from `%s'\n"), pathname);
@@ -380,15 +418,15 @@ void main(int argc, char *argv[])
       FILE* mapf;
       
       if (NULL == (mapf = findsfm(ufil, pathname, sizeof(pathname), stdin, NULL)))
-	  perror("findsfm"), exit (1);
+	perror("findsfm"), exit (1);
       
       if (no_act)
-	  fprintf (stderr, _("Would read screen-font map from %s.\n"), pathname);
+	fprintf (stderr, _("Would read screen-font map from %s.\n"), pathname);
       else if (verbose)
-	  fprintf (stderr, _("Reading screen-font map from %s.\n"), pathname);
+	fprintf (stderr, _("Reading screen-font map from %s.\n"), pathname);
 
       if (sfm_read_ascii (mapf, &ud, 512))
-	  perror ("sfm_read_ascii"), exit (EX_DATAERR);
+	perror ("sfm_read_ascii"), exit (EX_DATAERR);
     }
   
   if (fallback_files)
@@ -470,19 +508,19 @@ void main(int argc, char *argv[])
 	  memcpy (&ud, &new_ud, sizeof(struct unimapdesc));
 	}
       else
-	  fprintf (stderr, _("WARNING: No SFM found in file or kernel ?  Ignoring fallback file.\n"));
+	fprintf (stderr, _("WARNING: No SFM found in file or kernel ?  Ignoring fallback file.\n"));
     }
 	
   
   if (ud.entry_ct)
     {
       if (no_act)
-	  fprintf (stderr, _("Would set kernel SFM.\n"));
+	fprintf (stderr, _("Would set kernel SFM.\n"));
       else
 	{
 	  if (verbose) fprintf (stderr, _("Setting kernel SFM.\n"));
 	  if (set_kernel_sfm (fd, &ud))
-	      perror ("set_kernel_sfm"), exit (EX_OSERR);
+	    perror ("set_kernel_sfm"), exit (EX_OSERR);
 	}
     }
   
@@ -492,72 +530,67 @@ void main(int argc, char *argv[])
 
 simple_font* read_font (FILE* fontfile, int font_format, int requested_height)
 {
-  switch (font_format)
+  if (fff_is_simple_font (font_format))
     {
-    case FFF_PSF:
-    case FFF_BDF:
-    case FFF_RAW:
-      {
-	simple_font* font = read_simple_font (fontfile, font_format);
+      simple_font* font = read_simple_font (fontfile, font_format);
 	
-	if (requested_height && (requested_height != font->font.charheight))
-	  {
-	    free_simple_font (font);
-	    errno = EINVAL;
-	    return NULL;
-	  }
+      if (requested_height && (requested_height != font->font.charheight))
+	{
+	  free_simple_font (font);
+	  errno = EINVAL;
+	  return NULL;
+	}
 	
-	return font;
-      }
-      break;
+      return font;
+    }
 
-    case FFF_CP:
-    case FFF_XPSF:
-      {
-	font_group* fontgroup = read_font_group (fontfile, font_format);
-	simple_font* font;
-	int index;
+  else if (fff_is_font_group (font_format))
+    {
+      font_group* fontgroup = read_font_group (fontfile, font_format);
+      simple_font* font;
+      int index;
 
-	/* if only one font in group, do not require it on command-line */
-	if ((requested_height == 0) && (fontgroup->fontcount == 1))
-	  requested_height = fontgroup->fontcount;
+      /* if only one font in group, do not require it on command-line */
+      if ((requested_height == 0) && (fontgroup->fontcount == 1))
+	requested_height = fontgroup->fontcount;
 
 #if 1
-	fprintf (stderr, "Group has %d fonts:", fontgroup->fontcount);
-	for (index=0; index<fontgroup->fontcount; index++)
-	  fprintf (stderr, " %d", fontgroup->fonts[index].charheight);
-	fprintf (stderr, "\n");
+      fprintf (stderr, "Group has %d fonts:", fontgroup->fontcount);
+      for (index=0; index<fontgroup->fontcount; index++)
+	fprintf (stderr, " %d", fontgroup->fonts[index].charheight);
+      fprintf (stderr, "\n");
 #endif
 	
-	/* find requested_height in group */
-	for (index=0; index<fontgroup->fontcount; index++)
-	  if (fontgroup->fonts[index].charheight == requested_height)
-	    break;
+      /* find requested_height in group */
+      for (index=0; index<fontgroup->fontcount; index++)
+	if (fontgroup->fonts[index].charheight == requested_height)
+	  break;
 
-	if (requested_height == 0)
-	  {
-	    fprintf (stderr, "No font height specified\n");
-	    errno = EINVAL;
-	    return NULL;
-	  }
+      if (requested_height == 0)
+	{
+	  fprintf (stderr, "No font height specified\n");
+	  errno = EINVAL;
+	  return NULL;
+	}
 
-	if (index==fontgroup->fontcount)
-	  {
-	    errno = EINVAL;
-	    return NULL; /* height not in file */
-	  }
+      if (index==fontgroup->fontcount)
+	{
+	  errno = EINVAL;
+	  return NULL; /* height not in file */
+	}
 
-	font = (simple_font*)calloc (1, sizeof(simple_font));
-	if (!font) return NULL;
+      font = (simple_font*)calloc (1, sizeof(simple_font));
+      if (!font) return NULL;
 
-	memcpy (&font->sfm, &fontgroup->sfm, sizeof (struct unimapdesc));
-	memcpy (&font->font, &fontgroup->fonts[index], sizeof (struct cfontdesc));
+      memcpy (&font->sfm, &fontgroup->sfm, sizeof (struct unimapdesc));
+      memcpy (&font->font, &fontgroup->fonts[index], sizeof (struct cfontdesc));
 
-	free_font_group (fontgroup);
-	return font;
-      }
-
-    default:
+      free_font_group (fontgroup);
+      return font;
+    }
+  
+  else
+    {
       errno = EINVAL;
       return NULL;
     }
